package com.qmth.distributed.print.business.templete.importData;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysConfig;
import com.qmth.teachcloud.common.entity.TBTask;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.service.TBTaskService;
import com.qmth.teachcloud.common.util.FileStoreUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Async;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.text.MessageFormat;
import java.util.Date;
import java.util.Map;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: 异步导入模版
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
public abstract class AsyncImportTaskTemplete {

    private final static Logger log = LoggerFactory.getLogger(AsyncImportTaskTemplete.class);

    public static final String BEGIN_TITLE = "->开始准备处理导入的";
    public static final String EXCEPTION_TITLE = "->数据处理发生异常!";
    public static final String EXCEPTION_DATA = "错误信息:";
    public static final String FINISH_TITLE = "->数据处理结束，共处理了";
    public static final String FINISH_SIZE = "条数据";
    public static final String TXT_PREFIX = ".txt";
    public static final String EXCEPTION_CREATE_TXT_TITLE = "->创建导出日志时发生异常!";

    /**
     * 异步导入任务
     *
     * @param map
     * @return
     * @throws IOException
     */
    @Async
    public abstract Result importTask(Map<String, Object> map) throws IOException, Exception;

    /**
     * 获取上传的文件
     *
     * @param tbTask
     * @return
     * @throws IOException
     */
    public InputStream getUploadFileInputStream(TBTask tbTask) throws Exception {
        FileStoreUtil fileStoreUtil = SpringContextHolder.getBean(FileStoreUtil.class);
        JSONObject jsonObject = JSONObject.parseObject(tbTask.getImportFilePath());
        String path = (String) jsonObject.get(SystemConstant.PATH);
        String type = (String) jsonObject.get(SystemConstant.TYPE);
        UploadFileEnum uploadType = Enum.valueOf(UploadFileEnum.class, (String) jsonObject.get(SystemConstant.UPLOAD_TYPE));
        InputStream inputStream = null;
        if (Objects.equals(type, SystemConstant.OSS)) {
            inputStream = fileStoreUtil.ossDownloadIs(path, uploadType.getFssType());
        } else {
//            StringJoiner localPath = new StringJoiner("").add(SystemConstant.TEMP_FILES_DIR).add(File.separator).add(path);
            DictionaryConfig dictionaryConfig = SpringContextHolder.getBean(DictionaryConfig.class);
            if (Objects.nonNull(dictionaryConfig.fssLocalFileDomain()) && !StringUtils.isBlank(dictionaryConfig.fssLocalFileDomain().getConfig())) {
                inputStream = new FileInputStream(new File(path));
            } else {
                inputStream = fileStoreUtil.ossDownloadIs(path, uploadType.getFssType());
            }
//            inputStream = fileStoreUtil.ossDownloadIs(path, "localfile");
        }
        return inputStream;
    }

    /**
     * 创建txt文件
     *
     * @param tbTask
     */
    public void createTxt(TBTask tbTask) throws IOException {
        TBTaskService tbTaskService = SpringContextHolder.getBean(TBTaskService.class);
        File txtFileTemp = null;
        try {
            FileStoreUtil fileStoreUtil = SpringContextHolder.getBean(FileStoreUtil.class);
            DictionaryConfig dictionaryConfig = SpringContextHolder.getBean(DictionaryConfig.class);
            CommonCacheService commonCacheService = SpringContextHolder.getBean(CommonCacheService.class);
            boolean oss = dictionaryConfig.sysDomain().isOss();
            String ossStr = null;
            StringJoiner stringJoiner = new StringJoiner("");
            if (!oss && Objects.nonNull(dictionaryConfig.fssLocalFileDomain()) && !StringUtils.isBlank(dictionaryConfig.fssLocalFileDomain().getConfig())) {
                stringJoiner.add(dictionaryConfig.fssLocalFileDomain().getConfig()).add(File.separator);
            }
            stringJoiner = SystemConstant.getDirName(stringJoiner, UploadFileEnum.FILE, true);
            stringJoiner.add(SystemConstant.getNanoId()).add(TXT_PREFIX);

            String txtDirName = stringJoiner.toString();
            txtFileTemp = SystemConstant.getFileTempVar(TXT_PREFIX);
            SysConfig sysConfig = commonCacheService.addSysConfigCache(SystemConstant.SYS_CONFIG_KEY_CHARSETS);
            if (Objects.nonNull(sysConfig) && Objects.nonNull(sysConfig.getConfigValue())) {
                IOUtils.write(tbTask.getSummary().getBytes(sysConfig.getConfigValue()), new FileOutputStream(txtFileTemp));
            } else {
                IOUtils.write(tbTask.getSummary().getBytes(StandardCharsets.UTF_8), new FileOutputStream(txtFileTemp));
            }

            String txtFileMd5 = DigestUtils.md5Hex(new FileInputStream(txtFileTemp));
            if (oss) {
                ossStr = SystemConstant.OSS;
                fileStoreUtil.ossUpload(txtDirName, txtFileTemp, txtFileMd5, fileStoreUtil.getUploadEnumByPath(txtDirName).getFssType());
            } else {
                ossStr = SystemConstant.LOCAL;
                if (Objects.nonNull(dictionaryConfig.fssLocalFileDomain()) && !StringUtils.isBlank(dictionaryConfig.fssLocalFileDomain().getConfig())) {
                    fileStoreUtil.localUpload(txtDirName, new FileInputStream(txtFileTemp), txtFileMd5, LocalCatalogEnum.LOCAL_FILE);
                } else {
                    fileStoreUtil.ossUpload(txtDirName, txtFileTemp, txtFileMd5, fileStoreUtil.getUploadEnumByPath(txtDirName).getFssType());
                }
            }
            JSONObject json = new JSONObject();
            json.put(SystemConstant.PATH, stringJoiner.toString());
            json.put(SystemConstant.TYPE, ossStr);
            json.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);
            tbTask.setReportFilePath(json.toJSONString());
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            StringJoiner stringJoinerSummary = new StringJoiner("").add(tbTask.getSummary()).add("\n");
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN), EXCEPTION_CREATE_TXT_TITLE, EXCEPTION_DATA, e.getMessage()));

            String summary = stringJoinerSummary.toString();
            if (summary.length() >= 65535) {
                summary = "Data too long : " + summary.substring(0, 100) + "......" + summary.substring(summary.length() - 100);
            }

            tbTask.setSummary(summary);
            tbTask.setResult(TaskResultEnum.ERROR);
//            TBTask dbTask = tbTaskService.getById(tbTask.getId());
//            if (tbTask.getVersion() == dbTask.getVersion()) {
//                tbTask.setResetCount(new AtomicInteger(tbTask.getResetCount()).incrementAndGet());
//            }
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            if (Objects.nonNull(txtFileTemp)) {
                txtFileTemp.delete();
            }
            tbTask.setStatus(TaskStatusEnum.FINISH);
            if (tbTask.getType() == TaskTypeEnum.CREATE_PDF) {
                tbTask.setResultFilePath(tbTask.getImportFilePath());
                tbTask.setImportFileName(null);
                tbTask.setImportFilePath(null);
                tbTask.setErrorMessage(null);
            }
            tbTaskService.updateById(tbTask);
        }
    }
}
